/*
 * Copyright (C) 2010-2011 Freescale Semiconductor, Inc. All Rights Reserved.
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <linux/types.h>
#include <linux/sched.h>
#include <linux/delay.h>
#include <linux/pm.h>
#include <linux/interrupt.h>
#include <linux/irq.h>
#include <linux/init.h>
#include <linux/input.h>
#include <linux/nodemask.h>
#include <linux/clk.h>
#include <linux/platform_device.h>
#include <linux/fsl_devices.h>
#include <linux/spi/spi.h>
#include <linux/can/platform/flexcan.h>

#include <linux/spi/spi_gpio.h>
#include <linux/spi/ads7846.h>

#include <linux/i2c.h>
#include <linux/ata.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>
#include <linux/regulator/consumer.h>
#include <linux/pmic_external.h>
#include <linux/pmic_status.h>
#include <linux/ipu.h>
#include <linux/mxcfb.h>
#include <linux/pwm_backlight.h>
#include <linux/fec.h>
#include <linux/powerkey.h>
#include <linux/ahci_platform.h>
#include <linux/gpio_keys.h>
#include <linux/mfd/da9052/da9052.h>
#include <mach/common.h>
#include <mach/hardware.h>
#include <asm/irq.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/time.h>
#include <asm/mach/keypad.h>
#include <asm/mach/flash.h>
#include <mach/memory.h>
#include <mach/gpio.h>
#include <mach/mmc.h>
#include <mach/mxc_dvfs.h>
#include <mach/iomux-mx53.h>
#include <mach/i2c.h>
#include <mach/mxc_iim.h>
#include <mach/check_fuse.h>

#include "crm_regs.h"
#include "devices.h"
#include "usb.h"
#include "pmic.h"

/*!
 * @file mach-mx5/mx53_loco.c
 *
 * @brief This file contains MX53 loco board specific initialization routines.
 *
 * @ingroup MSL_MX53
 */

#if 0
/* MX53 LOCO GPIO PIN configurations */
#define NVDD_FAULT			(0*32 + 5)	/* GPIO1_5 */
#define FEC_INT				(1*32 + 4)	/* GPIO_2_4 */
#define MIC_DEC_B			(1*32 + 6)	/* GPIO_2_6 */
#define USER_UI1			(1*32 + 14)	/* GPIO_2_14 */
#define USER_UI2			(1*32 + 15)	/* GPIO_2_15 */
#define MX53_nONKEY			(0*32 + 8)	/* GPIO_1_8 */
#define DISP0_POWER_EN		(2*32 + 24)	/* GPIO_3_24 */
#define DISP0_DET_INT		(2*32 + 31)	/* GPIO_3_31 */
#define DISP0_RESET			(4*32 + 0)	/* GPIO_5_0 */
#define CSI0_RTSB			(5*32 + 9)	/* GPIO_6_9 */
#define CSI0_PWDN			(5*32 + 10)	/* GPIO_6_10 */
#define ACCL_EN				(5*32 + 14)	/* GPIO_6_14 */
#define ACCL_INT1_IN		(5*32 + 15)	/* GPIO_6_15 */
#define ACCL_INT2_IN		(5*32 + 16)	/* GPIO_6_16 */
#define LCD_BLT_EN			(6*32 + 2)	/* GPIO_7_2 */
#define FEC_RST				(6*32 + 6)	/* GPIO_7_6 */
#define USER_LED_EN			(6*32 + 7)	/* GPIO_7_7 */
#define NIRQ				(6*32 + 11)	/* GPIO7_11 */
#define MX53_LOCO_MC34708_IRQ_REVA    (4*32 + 30)	/* GPIO5_30 */
#define MX53_LOCO_MC34708_IRQ_REVB    (4*32 + 23)	/* GPIO5_23 */

#define MX53_OFFSET					(0x20000000)
#define TZIC_WAKEUP0_OFFSET         (0x0E00)
#define TZIC_WAKEUP1_OFFSET         (0x0E04)
#define TZIC_WAKEUP2_OFFSET         (0x0E08)
#define TZIC_WAKEUP3_OFFSET         (0x0E0C)
#define GPIO7_0_11_IRQ_BIT			(0x1<<11)

#endif

#define SD1_CD				(2*32 + 14)	 /* GPIO_3_14 */
#define OTG_VBUS            (1*32 + 24)	 /* GPIO_2_24 */
#define USB_PWREN			(1*32 + 29)	 /* GPIO_2_29 */
#define ADV_RST				(2*32 + 13)  /* GPIO_3_13 */
#define ADV_PD				(1*32 + 23)  /* GPIO_2_23 */
#define FEC_RST				(0*32 + 1)	/* GPIO_1_1 */

/* TS LVDS0 */
#define TS0_PENIRQ         (2*32 + 22)  /* GPIO_3_22 */
#define TS0_CS             (1*32 + 20)  /* GPIO_2_20 */ 
#define TS0_SCK            (1*32 + 16)  /* GPIO_2_16 */       
#define TS0_MOSI           (1*32 + 17)  /* GPIO_2_17 */
#define TS0_MISO           (1*32 + 22)  /* GPIO_2_22 */

/* TS LVDS1 */
#define TS1_PENIRQ         (2*32 + 19)  /* GPIO_3_19 */
#define TS1_CS             (1*32 + 10)  /* GPIO_2_10 */ 
#define TS1_SCK            (2*32 + 30)  /* GPIO_3_30 */       
#define TS1_MOSI           (1*32 + 15)  /* GPIO_2_15 */
#define TS1_MISO           (1*32 + 9)   /* GPIO_2_9 */

/* TS TFT */
#define TS2_PENIRQ           (1*32 + 6)   /* GPIO_2_6 */
#define TS2_CS               (1*32 + 8)   /* GPIO_2_8 */ 
#define TS2_SCK              (7)          /* GPIO_7 */       
#define TS2_MOSI             (1*32 + 3)   /* GPIO_2_3 */
#define TS2_MISO             (6*32 + 12)  /* GPIO_7_12 */


extern void pm_i2c_init(u32 base_addr);
static u32 mx53_loco_mc34708_irq;
static iomux_v3_cfg_t mx53_loco_pads[] = {
#if 0
	/* VGA */
	MX53_PAD_EIM_OE__IPU_DI1_PIN7,
	MX53_PAD_EIM_RW__IPU_DI1_PIN8,
	/* DISPLB */
	MX53_PAD_EIM_D20__IPU_SER_DISP0_CS,
	MX53_PAD_EIM_D21__IPU_DISPB0_SER_CLK,
	MX53_PAD_EIM_D22__IPU_DISPB0_SER_DIN,
	MX53_PAD_EIM_D23__IPU_DI0_D0_CS,
	/* DISP0_POWER_EN */
	MX53_PAD_EIM_D24__GPIO3_24,
	/* DISP0 DET INT */
	MX53_PAD_EIM_D31__GPIO3_31,
	/* PWM */
	MX53_PAD_GPIO_1__PWM2_PWMO,
	/* SPDIF */
	MX53_PAD_GPIO_7__SPDIF_PLOCK,
	MX53_PAD_GPIO_17__SPDIF_OUT1,
	/* GPIO */
	MX53_PAD_PATA_DA_1__GPIO7_7,
	MX53_PAD_PATA_DA_2__GPIO7_8,
	MX53_PAD_PATA_DATA5__GPIO2_5,
	MX53_PAD_PATA_DATA6__GPIO2_6,
	MX53_PAD_PATA_DATA14__GPIO2_14,
	MX53_PAD_PATA_DATA15__GPIO2_15,
	MX53_PAD_PATA_INTRQ__GPIO7_2,
	MX53_PAD_EIM_WAIT__GPIO5_0,
	MX53_PAD_NANDF_WP_B__GPIO6_9,
	MX53_PAD_NANDF_RB0__GPIO6_10,
	MX53_PAD_NANDF_CS1__GPIO6_14,
	MX53_PAD_NANDF_CS2__GPIO6_15,
	MX53_PAD_NANDF_CS3__GPIO6_16,
	MX53_PAD_GPIO_5__GPIO1_5,
	MX53_PAD_GPIO_16__GPIO7_11,
	MX53_PAD_GPIO_8__GPIO1_8,
#endif
	/* VGA by jury093 */
	MX53_PAD_EIM_OE__IPU_DI1_PIN7,
	MX53_PAD_EIM_RW__IPU_DI1_PIN8,

       /* CSI0 */
	MX53_PAD_CSI0_DAT12__IPU_CSI0_D_12,
	MX53_PAD_CSI0_DAT13__IPU_CSI0_D_13,
	MX53_PAD_CSI0_DAT14__IPU_CSI0_D_14,
	MX53_PAD_CSI0_DAT15__IPU_CSI0_D_15,
	MX53_PAD_CSI0_DAT16__IPU_CSI0_D_16,
	MX53_PAD_CSI0_DAT17__IPU_CSI0_D_17,
	MX53_PAD_CSI0_DAT18__IPU_CSI0_D_18,
	MX53_PAD_CSI0_DAT19__IPU_CSI0_D_19,
	MX53_PAD_CSI0_VSYNC__IPU_CSI0_VSYNC,
	MX53_PAD_CSI0_MCLK__IPU_CSI0_HSYNC,
	MX53_PAD_CSI0_PIXCLK__IPU_CSI0_PIXCLK,
	/* ADV_PD */
	MX53_PAD_EIM_CS0__GPIO2_23,
	/* ADV_RST */
	MX53_PAD_EIM_DA13__GPIO3_13,

	/* OTG_VBUS */
	MX53_PAD_EIM_CS1__GPIO2_24,

	/* USB_PWREN */
	MX53_PAD_EIM_EB1__GPIO2_29,

	/* FEC */
	MX53_PAD_FEC_MDC__FEC_MDC,
	MX53_PAD_FEC_MDIO__FEC_MDIO,
	MX53_PAD_FEC_REF_CLK__FEC_TX_CLK,
	MX53_PAD_FEC_RX_ER__FEC_RX_ER,
	MX53_PAD_FEC_CRS_DV__FEC_RX_DV,
	MX53_PAD_FEC_RXD1__FEC_RDATA_1,
	MX53_PAD_FEC_RXD0__FEC_RDATA_0,
	MX53_PAD_FEC_TX_EN__FEC_TX_EN,
	MX53_PAD_FEC_TXD1__FEC_TDATA_1,
	MX53_PAD_FEC_TXD0__FEC_TDATA_0,
	/* PHY CLK */
	MX53_PAD_GPIO_5__CCM_CLKO,
	/* PHY RESET */
	MX53_PAD_GPIO_1__GPIO1_1,

	/* CAN1 */
	MX53_PAD_PATA_INTRQ__CAN1_TXCAN,
	MX53_PAD_PATA_DIOR__CAN1_RXCAN,

	/* CAN2 */
	MX53_PAD_PATA_RESET_B__CAN2_TXCAN,
	MX53_PAD_PATA_IORDY__CAN2_RXCAN,

	/* AUDMUX4 */
	MX53_PAD_SD2_DATA0__AUDMUX_AUD4_RXD, 
	MX53_PAD_SD2_DATA1__AUDMUX_AUD4_TXFS,
	MX53_PAD_SD2_DATA2__AUDMUX_AUD4_TXD,
	MX53_PAD_SD2_DATA3__AUDMUX_AUD4_TXC,

	/* I2C1 */
	MX53_PAD_EIM_D21__I2C1_SCL,
	MX53_PAD_EIM_D28__I2C1_SDA,

	/* I2C2 */
	MX53_PAD_EIM_EB2__I2C2_SCL,
	MX53_PAD_KEY_ROW3__I2C2_SDA,

	/* SD1 */
	MX53_PAD_SD1_CMD__ESDHC1_CMD,
	MX53_PAD_SD1_CLK__ESDHC1_CLK,
	MX53_PAD_SD1_DATA0__ESDHC1_DAT0,
	MX53_PAD_SD1_DATA1__ESDHC1_DAT1,
	MX53_PAD_SD1_DATA2__ESDHC1_DAT2,
	MX53_PAD_SD1_DATA3__ESDHC1_DAT3,
	/* SD1_CD */
	MX53_PAD_EIM_DA14__GPIO3_14,

	/* LVDS */
	MX53_PAD_LVDS0_TX3_P__LDB_LVDS0_TX3,
	MX53_PAD_LVDS0_CLK_P__LDB_LVDS0_CLK,
	MX53_PAD_LVDS0_TX2_P__LDB_LVDS0_TX2,
	MX53_PAD_LVDS0_TX1_P__LDB_LVDS0_TX1,
	MX53_PAD_LVDS0_TX0_P__LDB_LVDS0_TX0,
	MX53_PAD_LVDS1_TX3_P__LDB_LVDS1_TX3,
	MX53_PAD_LVDS1_TX2_P__LDB_LVDS1_TX2,
	MX53_PAD_LVDS1_CLK_P__LDB_LVDS1_CLK,
	MX53_PAD_LVDS1_TX1_P__LDB_LVDS1_TX1,
	MX53_PAD_LVDS1_TX0_P__LDB_LVDS1_TX0,

	/* TS LVDS0 */
	MX53_PAD_EIM_A21__GPIO2_17,
	MX53_PAD_EIM_A16__GPIO2_22,
	MX53_PAD_EIM_A22__GPIO2_16,
	MX53_PAD_EIM_A18__GPIO2_20,
	MX53_PAD_EIM_D22__GPIO3_22,	

	/* TS LVDS1 */
	MX53_PAD_EIM_D19__GPIO3_19,	
	MX53_PAD_PATA_DATA10__GPIO2_10,
	MX53_PAD_PATA_DATA15__GPIO2_15,
	MX53_PAD_PATA_DATA9__GPIO2_9,
	MX53_PAD_EIM_D30__GPIO3_30,

	/* TS RGB */
	MX53_PAD_PATA_DATA8__GPIO2_8,
	MX53_PAD_PATA_DATA3__GPIO2_3,
	MX53_PAD_PATA_DATA6__GPIO2_6,
	MX53_PAD_GPIO_17__GPIO7_12,
	MX53_PAD_GPIO_7__GPIO1_7,

	/* UART1 */
	MX53_PAD_PATA_DIOW__UART1_TXD_MUX,
	MX53_PAD_PATA_DMACK__UART1_RXD_MUX,

	/* DISPLAY */
	MX53_PAD_DI0_DISP_CLK__IPU_DI0_DISP_CLK,
	MX53_PAD_DI0_PIN15__IPU_DI0_PIN15,
	MX53_PAD_DI0_PIN2__IPU_DI0_PIN2,
	MX53_PAD_DI0_PIN3__IPU_DI0_PIN3,
	MX53_PAD_DISP0_DAT0__IPU_DISP0_DAT_0,
	MX53_PAD_DISP0_DAT1__IPU_DISP0_DAT_1,
	MX53_PAD_DISP0_DAT2__IPU_DISP0_DAT_2,
	MX53_PAD_DISP0_DAT3__IPU_DISP0_DAT_3,
	MX53_PAD_DISP0_DAT4__IPU_DISP0_DAT_4,
	MX53_PAD_DISP0_DAT5__IPU_DISP0_DAT_5,
	MX53_PAD_DISP0_DAT6__IPU_DISP0_DAT_6,
	MX53_PAD_DISP0_DAT7__IPU_DISP0_DAT_7,
	MX53_PAD_DISP0_DAT8__IPU_DISP0_DAT_8,
	MX53_PAD_DISP0_DAT9__IPU_DISP0_DAT_9,
	MX53_PAD_DISP0_DAT10__IPU_DISP0_DAT_10,
	MX53_PAD_DISP0_DAT11__IPU_DISP0_DAT_11,
	MX53_PAD_DISP0_DAT12__IPU_DISP0_DAT_12,
	MX53_PAD_DISP0_DAT13__IPU_DISP0_DAT_13,
	MX53_PAD_DISP0_DAT14__IPU_DISP0_DAT_14,
	MX53_PAD_DISP0_DAT15__IPU_DISP0_DAT_15,
	MX53_PAD_DISP0_DAT16__IPU_DISP0_DAT_16,
	MX53_PAD_DISP0_DAT17__IPU_DISP0_DAT_17,
	MX53_PAD_DISP0_DAT18__IPU_DISP0_DAT_18,
	MX53_PAD_DISP0_DAT19__IPU_DISP0_DAT_19,
	MX53_PAD_DISP0_DAT20__IPU_DISP0_DAT_20,
	MX53_PAD_DISP0_DAT21__IPU_DISP0_DAT_21,
	MX53_PAD_DISP0_DAT22__IPU_DISP0_DAT_22,
	MX53_PAD_DISP0_DAT23__IPU_DISP0_DAT_23,

	/* Audio CLK*/
	MX53_PAD_GPIO_0__CCM_SSI_EXT1_CLK,
};

static iomux_v3_cfg_t mx53_nand_pads[] = {
	MX53_PAD_NANDF_CLE__EMI_NANDF_CLE,
	MX53_PAD_NANDF_ALE__EMI_NANDF_ALE,
	MX53_PAD_NANDF_WP_B__EMI_NANDF_WP_B,
	MX53_PAD_NANDF_WE_B__EMI_NANDF_WE_B,
	MX53_PAD_NANDF_RE_B__EMI_NANDF_RE_B,
	MX53_PAD_NANDF_RB0__EMI_NANDF_RB_0,
	MX53_PAD_NANDF_CS0__EMI_NANDF_CS_0,
	MX53_PAD_NANDF_CS1__EMI_NANDF_CS_1,
	MX53_PAD_NANDF_CS2__EMI_NANDF_CS_2,
	MX53_PAD_NANDF_CS3__EMI_NANDF_CS_3,
	MX53_PAD_EIM_DA0__EMI_NAND_WEIM_DA_0,
	MX53_PAD_EIM_DA1__EMI_NAND_WEIM_DA_1,
	MX53_PAD_EIM_DA2__EMI_NAND_WEIM_DA_2,
	MX53_PAD_EIM_DA3__EMI_NAND_WEIM_DA_3,
	MX53_PAD_EIM_DA4__EMI_NAND_WEIM_DA_4,
	MX53_PAD_EIM_DA5__EMI_NAND_WEIM_DA_5,
	MX53_PAD_EIM_DA6__EMI_NAND_WEIM_DA_6,
	MX53_PAD_EIM_DA7__EMI_NAND_WEIM_DA_7,
};

#if defined(CONFIG_TOUCHSCREEN_ADS7846)
struct spi_gpio_platform_data sk_spi0_gpio_platform_data = {
	.sck = TS0_SCK,
	.mosi = TS0_MOSI,
	.miso = TS0_MISO,
	.num_chipselect = 1,
};

struct spi_gpio_platform_data sk_spi1_gpio_platform_data = {
	.sck = TS1_SCK,
	.mosi = TS1_MOSI,
	.miso = TS1_MISO,
	.num_chipselect = 1,
};

struct spi_gpio_platform_data sk_spi2_gpio_platform_data = {
	.sck = TS2_SCK,
	.mosi = TS2_MOSI,
	.miso = TS2_MISO,
	.num_chipselect = 1,
};

struct platform_device sk_spi0_gpio_device = {
	.name = "spi_gpio",
	.id = 0,
	.dev = {
		.platform_data = &sk_spi0_gpio_platform_data,
	},
};

struct platform_device sk_spi1_gpio_device = {
	.name = "spi_gpio",
	.id = 1,
	.dev = {
		.platform_data = &sk_spi1_gpio_platform_data,
	},
};

struct platform_device sk_spi2_gpio_device = {
	.name = "spi_gpio",
	.id = 2,
	.dev = {
		.platform_data = &sk_spi2_gpio_platform_data,
	},
};

static int ads7846_pendown_state0(void)
{
    /* Touchscreen PENIRQ */
	return !gpio_get_value(TS0_PENIRQ);	  
}

static int ads7846_pendown_state1(void)
{
    /* Touchscreen PENIRQ */
	return !gpio_get_value(TS1_PENIRQ);	  
}

static int ads7846_pendown_state2(void)
{
    /* Touchscreen PENIRQ */
	return !gpio_get_value(TS2_PENIRQ);	  
}

static struct ads7846_platform_data ads_info0 = {
    .model                  = 7843,
    .x_min                  = 150,
    .x_max                  = 3830,
    .y_min                  = 190,
    .y_max                  = 3830,
    .vref_delay_usecs       = 100,
    .x_plate_ohms           = 450,
    .y_plate_ohms           = 250,
    .pressure_max           = 15000,
    .debounce_max           = 5,
    .debounce_rep           = 0,
    .debounce_tol           = 10,
	.get_pendown_state = ads7846_pendown_state0,
};

static struct ads7846_platform_data ads_info1 = {
    .model                  = 7843,
    .x_min                  = 150,
    .x_max                  = 3830,
    .y_min                  = 190,
    .y_max                  = 3830,
    .vref_delay_usecs       = 100,
    .x_plate_ohms           = 450,
    .y_plate_ohms           = 250,
    .pressure_max           = 15000,
    .debounce_max           = 5,
    .debounce_rep           = 0,
    .debounce_tol           = 10,
	.get_pendown_state = ads7846_pendown_state1,
};

static struct ads7846_platform_data ads_info2 = {
    .model                  = 7843,
    .x_min                  = 150,
    .x_max                  = 3830,
    .y_min                  = 190,
    .y_max                  = 3830,
    .vref_delay_usecs       = 100,
    .x_plate_ohms           = 450,
    .y_plate_ohms           = 250,
    .pressure_max           = 15000,
    .debounce_max           = 1,
    .debounce_rep           = 0,
    .debounce_tol           = 10,
	.get_pendown_state = ads7846_pendown_state2,
};

static struct spi_board_info mxc_ts_device[] __initdata = {
	{
	 .modalias = "ads7846",
	 .max_speed_hz = 1000 * 1000,  /* max spi clock (SCK) speed in HZ */
	 .bus_num = 0,
	 .chip_select = 0,
	 .controller_data = (void *)TS0_CS,
	 .platform_data = &ads_info0,
	 .irq = gpio_to_irq(TS0_PENIRQ),
	},
	{
	 .modalias = "ads7846",
	 .max_speed_hz = 1000 * 1000,  /* max spi clock (SCK) speed in HZ */
	 .bus_num = 1,
	 .chip_select = 0,
	 .controller_data = (void *)TS1_CS,
	 .platform_data = &ads_info1,
	 .irq = gpio_to_irq(TS1_PENIRQ),
	},	
	{
	 .modalias = "ads7846",
	 .max_speed_hz = 1000 * 1000,  /* max spi clock (SCK) speed in HZ */
	 .bus_num = 2,
	 .chip_select = 0,
	 .controller_data = (void *)TS2_CS,
	 .platform_data = &ads_info2,
	 .irq = gpio_to_irq(TS2_PENIRQ),
	},	
	
#if 0	
	{
	 .modalias = "spidev",
	 .max_speed_hz = 1000 * 1000,  /* max spi clock (SCK) speed in HZ */
	 .bus_num = 0,
	 .chip_select = 0,
	 .controller_data = (void *)TS_CS,
	},
#endif
};
#endif



static struct fb_videomode video_modes[] = {
//	{
	 /* 800x480 @ 57 Hz , pixel clk @ 27MHz */
//	 "NVGA", 30, 800, 480, /*74074*/ /*59700*/ 37037, 40, 60, 10, 10, 20, 10,
//	 0,
//	 FB_VMODE_NONINTERLACED,
//	 FB_MODE_IS_DETAILED,},
	{
	 /* 480x272 for SK-MI0430FT_plug */
	 "SK-480x272", 60, 480, 272, /*100000*/ 150000, 40, 60, 10, 10, 20, 10,
	 FB_SYNC_CLK_LAT_FALL,
	 FB_VMODE_NONINTERLACED,
	 0,},
	{
	 /* 1280x800 for LVDS channels by Jury093*/
	 "SK-1280x800-LVDS", 60, 1280, 800, 15385, 220, 40, 21, 7, 60, 10, 0,
	 FB_VMODE_NONINTERLACED,
	 0,},

//	{
	 /* 1920x1080 for HDMI, Attention!!! must be turn on only DI0 !!! */
//	 "SK-1920x1080-HDMI", 60, 1920, 1080, 7692, 100, 40, 30, 3, 10, 2, 0,
/*	 FB_VMODE_NONINTERLACED,
	 FB_MODE_IS_DETAILED,},
	{*/
	 /* 1024x768 for TFT parallel channel and HDMI */
/*	 "SK-1024x768-HDMI", 60, 1024, 768, 15385, 220, 40, 21, 7, 60, 10,
	 FB_SYNC_CLK_LAT_FALL,
	 FB_VMODE_NONINTERLACED,
	 0,},
	{*/
	 /* 800x480 for LVDS channels */
/*	 "SK-800x480-LVDS", 60, 800, 480, 15385, 220, 40, 21, 7, 60, 10, 0,
	 FB_VMODE_NONINTERLACED,
	 0,},
	{*/
	 /* 800x480 for TFT parallel channel and HDMI */
/*	 "SK-800x480-TFT", 60, 800, 480, 15385 * 2, 220, 40, 21, 7, 60, 10,
	 FB_SYNC_CLK_LAT_FALL,
	 FB_VMODE_NONINTERLACED,
	 0,},*/
//	{
	 /* 800x480 @ 57 Hz , pixel clk @ 27MHz */
/*	 "CLAA-WVGA", 30, 800, 480, 37037, 40, 60, 10, 10, 20, 10,
	 0,
	 FB_VMODE_NONINTERLACED,
	 0,},
	{*/
	 /* 800x480 @ 60 Hz , pixel clk @ 32MHz */
/*	 "SEIKO-WVGA", 60, 800, 480, 29850, 89, 164, 23, 10, 10, 10,
	 FB_SYNC_CLK_LAT_FALL,
	 FB_VMODE_NONINTERLACED,
	 0,},
	{*/
	/* 1600x1200 @ 60 Hz 162M pixel clk*/
/*	"UXGA", 60, 1600, 1200, 6172,
	304, 64,
	1, 46,
	192, 3,
	FB_SYNC_HOR_HIGH_ACT|FB_SYNC_VERT_HIGH_ACT,
	FB_VMODE_NONINTERLACED,
	0,},*/
};

static struct platform_pwm_backlight_data mxc_pwm_backlight_data = {
	.pwm_id = 1,
	.max_brightness = 255,
	.dft_brightness = 128,
	.pwm_period_ns = 50000,
};

static void flexcan_xcvr_enable(int enable)
{

}

static struct flexcan_platform_data flexcan0_data = {
	.transceiver_switch = flexcan_xcvr_enable
};

static struct flexcan_platform_data flexcan1_data = {
	.transceiver_switch = flexcan_xcvr_enable
};

extern void mx5_ipu_reset(void);
static struct mxc_ipu_config mxc_ipu_data = {
	.rev = 3,
	.reset = mx5_ipu_reset,
};

extern void mx5_vpu_reset(void);
static struct mxc_vpu_platform_data mxc_vpu_data = {
	.iram_enable = true,
	.iram_size = 0x14000,
	.reset = mx5_vpu_reset,
};

static inline void mx53_loco_fec_init(void)
{
	struct clk *cko1_clk;
	struct clk *pll2_sw_clk;
	int rate;
	int ret;

	cko1_clk = clk_get(NULL, "cko1");
	if (IS_ERR(cko1_clk)) {
		printk("Error: can't find cko1_clk\n");
		return;
	}

	pll2_sw_clk = clk_get(NULL, "pll2");
	if (IS_ERR(pll2_sw_clk)) {
		printk("Error: can't find pll2_sw_clk\n");
		return;
	}

	clk_set_parent(cko1_clk, pll2_sw_clk);

	rate = clk_round_rate(cko1_clk, 50000000);
	if (rate != 50000000) {
		printk("Error: FEC PHY freq %d out of range\n", rate);
		clk_put(cko1_clk);
		return;
	} else 
		printk("FEC PHY freq %d\n", rate);
	
	clk_set_rate(cko1_clk, rate);
	clk_enable(cko1_clk);

	/* reset FEC PHY */
	ret = gpio_request(FEC_RST, "fec-phy-reset");
	if (ret) {
		pr_err("failed to get GPIO FEC_PHY_RST: %d\n", ret);
		return;
	}
	gpio_direction_output(FEC_RST, 0);
	msleep(1);
	gpio_set_value(FEC_RST, 1);
}

static struct fec_platform_data fec_data = {
	.phy = PHY_INTERFACE_MODE_RMII,
};

static struct mxc_dvfs_platform_data dvfs_core_data = {
	.clk1_id = "cpu_clk",
	.clk2_id = "gpc_dvfs_clk",
	.gpc_cntr_offset = MXC_GPC_CNTR_OFFSET,
	.gpc_vcr_offset = MXC_GPC_VCR_OFFSET,
	.ccm_cdcr_offset = MXC_CCM_CDCR_OFFSET,
	.ccm_cacrr_offset = MXC_CCM_CACRR_OFFSET,
	.ccm_cdhipr_offset = MXC_CCM_CDHIPR_OFFSET,
	.prediv_mask = 0x1F800,
	.prediv_offset = 11,
	.prediv_val = 3,
	.div3ck_mask = 0xE0000000,
	.div3ck_offset = 29,
	.div3ck_val = 2,
	.emac_val = 0x08,
	.upthr_val = 25,
	.dnthr_val = 9,
	.pncthr_val = 33,
	.upcnt_val = 10,
	.dncnt_val = 10,
	.delay_time = 30,
};

static struct mxc_bus_freq_platform_data bus_freq_data;

static struct tve_platform_data tve_data = {
	.boot_enable = MXC_TVE_VGA,
};

static struct ldb_platform_data ldb_data = {
	.ext_ref = 1,
	.boot_enable = MXC_LDBDI1,
//	MXC_LDBDI0 | MXC_LDBDI1,
};

static void mxc_iim_enable_fuse(void)
{
	u32 reg;

	if (!ccm_base)
		return;

	/* enable fuse blown */
	reg = readl(ccm_base + 0x64);
	reg |= 0x10;
	writel(reg, ccm_base + 0x64);
}

static void mxc_iim_disable_fuse(void)
{
	u32 reg;

	if (!ccm_base)
		return;
	/* enable fuse blown */
	reg = readl(ccm_base + 0x64);
	reg &= ~0x10;
	writel(reg, ccm_base + 0x64);
}

static struct mxc_iim_data iim_data = {
	.bank_start = MXC_IIM_MX53_BANK_START_ADDR,
	.bank_end   = MXC_IIM_MX53_BANK_END_ADDR,
	.enable_fuse = mxc_iim_enable_fuse,
	.disable_fuse = mxc_iim_disable_fuse,
};

static struct resource mxcfb_resources[] = {
	[0] = {
	       .flags = IORESOURCE_MEM,
	       },
};

static struct mxc_fb_platform_data fb_data[] = {
	{
	 .interface_pix_fmt = IPU_PIX_FMT_RGB24,
	 .mode_str = "SK-1280x800-LVDS",
	 .mode = video_modes,
//	 .mode = 1,
	 .num_modes = ARRAY_SIZE(video_modes),
	 },
	{
	 .interface_pix_fmt = IPU_PIX_FMT_RGB24,
	 .mode_str = "CLAA-WVGA",
//	 .mode_str = "SK-480x272",
	 .mode = video_modes,
//	 .mode = 0,
	 .num_modes = ARRAY_SIZE(video_modes),
	 },
};

extern int primary_di;
static int __init mxc_init_fb(void)
{
//	primary_di = 0;

	if (primary_di) {
		printk(KERN_INFO "DI1 is primary\n");
		/* DI1 -> DP-BG channel: */
		mxc_fb_devices[1].num_resources = ARRAY_SIZE(mxcfb_resources);
		mxc_fb_devices[1].resource = mxcfb_resources;
		mxc_register_device(&mxc_fb_devices[1], &fb_data[1]);

		/* DI0 -> DC channel: */
		mxc_register_device(&mxc_fb_devices[0], &fb_data[0]);
	} else {
		printk(KERN_INFO "DI0 is primary\n");

		/* DI0 -> DP-BG channel: */
		mxc_fb_devices[0].num_resources = ARRAY_SIZE(mxcfb_resources);
		mxc_fb_devices[0].resource = mxcfb_resources;
		mxc_register_device(&mxc_fb_devices[0], &fb_data[0]);

		/* DI1 -> DC channel: */
		mxc_register_device(&mxc_fb_devices[1], &fb_data[1]);
	}

	/*
	 * DI0/1 DP-FG channel:
	 */
	mxc_register_device(&mxc_fb_devices[2], NULL);

	return 0;
}
device_initcall(mxc_init_fb);

static void adv7180_pwdn(int pwdn)
{
	if (pwdn)
		gpio_set_value(ADV_PD, 0);
	else
		gpio_set_value(ADV_PD, 1);
}
        
static struct mxc_tvin_platform_data adv7180_data = {
	.dvddio_reg = NULL,
	.dvdd_reg = NULL,
	.avdd_reg = NULL,
	.pvdd_reg = NULL,
	.pwdn = adv7180_pwdn,
	.reset = NULL,
	.cvbs = 1,
};

static struct imxi2c_platform_data mxci2c_data = {
	.bitrate = 100000,
};

static struct i2c_board_info mxc_i2c0_board_info[] __initdata = {
/*	{
	.type = "adv7180",
	.addr = 0x21,
	.platform_data = (void *)&adv7180_data,
	},*/
};

static struct i2c_board_info mxc_i2c1_board_info[] __initdata = {
	{
	.type = "adv7180",
	.addr = 0x21,
	.platform_data = (void *)&adv7180_data,
	},
	{
	.type = "tlv320aic23",
	.addr = 0x1a,
	 },
/*	{
	.type = "ds1338",
	.addr = 0x68,
	 },	 */
	 {
	 I2C_BOARD_INFO("ds1338", 0x68),
//	 .type	= "ds1338",
	 },
};

static unsigned int sdhc_get_card_det_status(struct device *dev)
{
	return gpio_get_value(SD1_CD);
}

static struct mxc_mmc_platform_data mmc1_data = {
	.ocr_mask = MMC_VDD_27_28 | MMC_VDD_28_29 | MMC_VDD_29_30
		| MMC_VDD_31_32,
	.caps = MMC_CAP_4_BIT_DATA,
	.min_clk = 400000,
	.max_clk = 50000000,
	.card_inserted_state = 0,
	.status = sdhc_get_card_det_status,
	.clock_mmc = "esdhc_clk",
	.power_mmc = NULL,
};

static int mxc_sgtl5000_init(void);

static struct mxc_audio_platform_data sgtl5000_data = {
	.ssi_num = 1,
	.src_port = 2,
	.ext_port = 4,
	.init = mxc_sgtl5000_init,
	.ext_ram_rx = 1,
};

static int mxc_sgtl5000_init(void)
{
	struct clk *ssi_ext1;
	int rate;

	ssi_ext1 = clk_get(NULL, "ssi_ext1_clk");
	if (IS_ERR(ssi_ext1))
			return -1;

	rate = clk_round_rate(ssi_ext1, 12000000);
	if (rate < 8000000 || rate > 27000000) {
			printk(KERN_ERR "Error: SGTL5000 mclk freq %d out of range!\n",
				   rate);
			clk_put(ssi_ext1);
			return -1;
	}

	clk_set_rate(ssi_ext1, rate);
	clk_enable(ssi_ext1);
	sgtl5000_data.sysclk = rate;

	return 0;
}

static struct platform_device mxc_sgtl5000_device = {
	.name = "imx-3stack-sgtl5000",
};

/* NAND Flash Partitions */
#ifdef CONFIG_MTD_PARTITIONS
static struct mtd_partition nand_flash_partitions[] = {
/* MX53 ROM require the boot FCB/DBBT support which need
 * more space to store such info on NAND boot partition.
 * 16M should cover all kind of NAND boot support on MX53.
 */
	{
	 .name = "bootloader and kernel",
	 .offset = 0,
	 .size = 32 * 1024 * 1024},
	{
	 .name = "nand.rootfs",
	 .offset = MTDPART_OFS_APPEND,
	 .size = MTDPART_SIZ_FULL},
};
#endif

static int nand_init(void)
{
	u32 i, reg;
	void __iomem *base;

	#define M4IF_GENP_WEIM_MM_MASK          0x00000001
	#define WEIM_GCR2_MUX16_BYP_GRANT_MASK  0x00001000

	base = ioremap(MX53_BASE_ADDR(M4IF_BASE_ADDR), SZ_4K);
	reg = __raw_readl(base + 0xc);
	reg &= ~M4IF_GENP_WEIM_MM_MASK;
	__raw_writel(reg, base + 0xc);

	iounmap(base);

	base = ioremap(MX53_BASE_ADDR(WEIM_BASE_ADDR), SZ_4K);
	for (i = 0x4; i < 0x94; i += 0x18) {
		reg = __raw_readl((u32)base + i);
		reg &= ~WEIM_GCR2_MUX16_BYP_GRANT_MASK;
		__raw_writel(reg, (u32)base + i);
	}

	iounmap(base);

	return 0;
}

static struct flash_platform_data mxc_nand_data = {
#ifdef CONFIG_MTD_PARTITIONS
	.parts = nand_flash_partitions,
	.nr_parts = ARRAY_SIZE(nand_flash_partitions),
#endif
	.width = 1,
	.init = nand_init,
};

static struct mxc_asrc_platform_data mxc_asrc_data = {
	.channel_bits = 4,
	.clk_map_ver = 2,
};

static struct mxc_spdif_platform_data mxc_spdif_data = {
	.spdif_tx = 1,
	.spdif_rx = 0,
	.spdif_clk_44100 = -1,	/* Souce from CKIH1 for 44.1K */
	/* Source from CCM spdif_clk (24M) for 48k and 32k
	 * It's not accurate
	 */
	.spdif_clk_48000 = 1,
	.spdif_clkid = 0,
	.spdif_clk = NULL,	/* spdif bus clk */
};

static struct mxc_audio_platform_data spdif_audio_data = {
	.ext_ram_tx = 1,
};

static struct platform_device mxc_spdif_audio_device = {
	.name = "imx-spdif-audio-device",
};

static void mx53_gpio_usbotg_driver_vbus(bool on)
{
	if (on)
		gpio_set_value(OTG_VBUS, 1);
	else
		gpio_set_value(OTG_VBUS, 0);
}

static void mx53_loco_usbh1_vbus(bool on)
{
	if (on)
		gpio_set_value(USB_PWREN, 1);
	else
		gpio_set_value(USB_PWREN, 0);
}

#if defined(CONFIG_KEYBOARD_GPIO) || defined(CONFIG_KEYBOARD_GPIO_MODULE)
#define GPIO_BUTTON(gpio_num, ev_code, act_low, descr, wake)	\
{								\
	.gpio		= gpio_num,				\
	.type		= EV_KEY,				\
	.code		= ev_code,				\
	.active_low	= act_low,				\
	.desc		= "btn " descr,			\
	.wakeup		= wake,					\
}

static struct gpio_keys_button loco_buttons[] = {
	GPIO_BUTTON(MX53_nONKEY, KEY_POWER, 1, "power", 0),
	GPIO_BUTTON(USER_UI1, KEY_VOLUMEUP, 1, "volume-up", 0),
	GPIO_BUTTON(USER_UI2, KEY_VOLUMEDOWN, 1, "volume-down", 0),
};

static struct gpio_keys_platform_data loco_button_data = {
	.buttons	= loco_buttons,
	.nbuttons	= ARRAY_SIZE(loco_buttons),
};

static struct platform_device loco_button_device = {
	.name		= "gpio-keys",
	.id		= -1,
	.num_resources  = 0,
	.dev		= {
		.platform_data = &loco_button_data,
	}
};

static void __init loco_add_device_buttons(void)
{
	platform_device_register(&loco_button_device);
}
#else
static void __init loco_add_device_buttons(void) {}
#endif

#if 0
static void mxc_register_powerkey(pwrkey_callback pk_cb)
{
	pmic_event_callback_t power_key_event;

	power_key_event.param = (void *)1;
	power_key_event.func = (void *)pk_cb;
	pmic_event_subscribe(EVENT_PWRONI, power_key_event);
}

static int mxc_pwrkey_getstatus(int id)
{
	int sense;

	pmic_read_reg(REG_INT_SENSE1, &sense, 0xffffffff);
	if (sense & (1 << 3))
		return 0;

	return 1;
}

static struct power_key_platform_data pwrkey_data = {
	.key_value = KEY_F4,
	.register_pwrkey = mxc_register_powerkey,
	.get_key_status = mxc_pwrkey_getstatus,
};
#endif
/*!
 * Board specific fixup function. It is called by \b setup_arch() in
 * setup.c file very early on during kernel starts. It allows the user to
 * statically fill in the proper values for the passed-in parameters. None of
 * the parameters is used currently.
 *
 * @param  desc         pointer to \b struct \b machine_desc
 * @param  tags         pointer to \b struct \b tag
 * @param  cmdline      pointer to the command line
 * @param  mi           pointer to \b struct \b meminfo
 */
static void __init fixup_mxc_board(struct machine_desc *desc, struct tag *tags,
				   char **cmdline, struct meminfo *mi)
{
	struct tag *t;
	struct tag *mem_tag = 0;
	int total_mem = SZ_256M;
	int left_mem = 0;
	int gpu_mem = 0; //SZ_128M;
	int fb_mem = SZ_32M;
	char *str;

	mxc_set_cpu_type(MXC_CPU_MX53);

	for_each_tag(mem_tag, tags) {
		if (mem_tag->hdr.tag == ATAG_MEM) {
			total_mem = mem_tag->u.mem.size;
			break;
		}
	}

	for_each_tag(t, tags) {
		if (t->hdr.tag == ATAG_CMDLINE) {
			str = t->u.cmdline.cmdline;
			str = strstr(str, "mem=");
			if (str != NULL) {
				str += 4;
				left_mem = memparse(str, &str);
			}

			str = t->u.cmdline.cmdline;
			str = strstr(str, "gpu_nommu");
			if (str != NULL)
				gpu_data.enable_mmu = 0;

			str = t->u.cmdline.cmdline;
			str = strstr(str, "gpu_memory=");
			if (str != NULL) {
				str += 11;
				gpu_mem = memparse(str, &str);
			}

			break;
		}
	}

	if (gpu_data.enable_mmu)
		gpu_mem = 0;

	if (left_mem == 0 || left_mem > total_mem)
		left_mem = total_mem - gpu_mem - fb_mem;

	if (mem_tag) {
		fb_mem = total_mem - left_mem - gpu_mem;
		if (fb_mem < 0) {
			gpu_mem = total_mem - left_mem;
			fb_mem = 0;
		}
		mem_tag->u.mem.size = left_mem;

		/*reserve memory for gpu*/
		if (!gpu_data.enable_mmu) {
			gpu_device.resource[5].start =
				mem_tag->u.mem.start + left_mem;
			gpu_device.resource[5].end =
				gpu_device.resource[5].start + gpu_mem - 1;
		}
#if defined(CONFIG_FB_MXC_SYNC_PANEL) || \
	defined(CONFIG_FB_MXC_SYNC_PANEL_MODULE)
		if (fb_mem) {
			mxcfb_resources[0].start =
				gpu_data.enable_mmu ?
				mem_tag->u.mem.start + left_mem :
				gpu_device.resource[5].end + 1;
			mxcfb_resources[0].end =
				mxcfb_resources[0].start + fb_mem - 1;
		} else {
			mxcfb_resources[0].start = 0;
			mxcfb_resources[0].end = 0;
		}
#endif
	}
}

static void __init mx53_loco_io_init(void)
{
	mxc_iomux_v3_setup_multiple_pads(mx53_loco_pads,
					ARRAY_SIZE(mx53_loco_pads));

	mxc_iomux_v3_setup_multiple_pads(mx53_nand_pads,
					ARRAY_SIZE(mx53_nand_pads));

	/* SD1 */
	gpio_request(SD1_CD, "sd1-cd");
	gpio_direction_input(SD1_CD);

#if defined(CONFIG_TOUCHSCREEN_ADS7846)
	gpio_request(TS0_PENIRQ, "ts0-penirq");
	gpio_direction_input(TS0_PENIRQ);

	gpio_request(TS1_PENIRQ, "ts1-penirq");
	gpio_direction_input(TS1_PENIRQ);

	gpio_request(TS2_PENIRQ, "ts2-penirq");
	gpio_direction_input(TS2_PENIRQ);
#endif
	/* ADV7180 */
	gpio_request(ADV_PD, "adv-pd");
	gpio_request(ADV_RST, "adv-rst");
	gpio_direction_output(ADV_PD, 0);
	gpio_direction_output(ADV_RST, 1);

	/* USB PWR enable */
	gpio_request(USB_PWREN, "usb-pwr");
	gpio_direction_output(USB_PWREN, 0);

	/* Config GPIO for OTG VBus */
	gpio_request(OTG_VBUS, "otg-vbus");
	gpio_direction_output(OTG_VBUS, 0);

#if 0
	/* LCD panel power enable */
	gpio_request(DISP0_POWER_EN, "disp0-power-en");
	gpio_direction_output(DISP0_POWER_EN, 1);
#endif
}

/*!
 * Board specific initialization.
 */
static void __init mxc_board_init(void)
{

//	iomux_v3_cfg_t mc34708_int = MX53_PAD_CSI0_DAT12__GPIO5_30;
//	iomux_v3_cfg_t da9052_csi0_d12;
	
	mx53_loco_fec_init();

	mxc_ipu_data.di_clk[0] = clk_get(NULL, "ipu_di0_clk");
	mxc_ipu_data.di_clk[1] = clk_get(NULL, "ipu_di1_clk");
	mxc_ipu_data.csi_clk[0] = clk_get(NULL, "ssi_ext1_clk");
	mxc_spdif_data.spdif_core_clk = clk_get(NULL, "spdif_xtal_clk");
	clk_put(mxc_spdif_data.spdif_core_clk);

	mxcsdhc1_device.resource[2].start = gpio_to_irq(SD1_CD);
	mxcsdhc1_device.resource[2].end = gpio_to_irq(SD1_CD);

	mxc_cpu_common_init();
	mx53_loco_io_init();

	mxc_register_device(&mxc_dma_device, NULL);
	mxc_register_device(&mxc_wdt_device, NULL);
	mxc_register_device(&mxci2c_devices[0], &mxci2c_data);
	mxc_register_device(&mxci2c_devices[1], &mxci2c_data);
	mxc_register_device(&mxci2c_devices[2], &mxci2c_data);

#if 0
    if (board_is_mx53_loco_mc34708()) {
		/* set pmic INT gpio pin */
		if (board_is_rev(BOARD_REV_2)) {/*Board rev A*/
			mc34708_int = MX53_PAD_CSI0_DAT12__GPIO5_30;
			mx53_loco_mc34708_irq = MX53_LOCO_MC34708_IRQ_REVA;
		} else if (board_is_rev(BOARD_REV_4)) {/*Board rev B*/
			mc34708_int = MX53_PAD_CSI0_DAT5__GPIO5_23;
			mx53_loco_mc34708_irq = MX53_LOCO_MC34708_IRQ_REVB;
		}
		mxc_iomux_v3_setup_pad(mc34708_int);
		gpio_request(mx53_loco_mc34708_irq, "pmic-int");
		gpio_direction_input(mx53_loco_mc34708_irq);
		mx53_loco_init_mc34708(mx53_loco_mc34708_irq);
		dvfs_core_data.reg_id = "SW1A";
		tve_data.dac_reg = "VDAC";
		bus_freq_data.gp_reg_id = "SW1A";
		bus_freq_data.lp_reg_id = "SW2";
		mxc_register_device(&mxc_powerkey_device, &pwrkey_data);
	}
    else {
		da9052_csi0_d12 = MX53_PAD_CSI0_DAT12__IPU_CSI0_D_12;
		mxc_iomux_v3_setup_pad(da9052_csi0_d12);
		mx53_loco_init_da9052();
		dvfs_core_data.reg_id = "DA9052_BUCK_CORE";
		tve_data.dac_reg = "DA9052_LDO7";
		bus_freq_data.gp_reg_id = "DA9052_BUCK_CORE";
		bus_freq_data.lp_reg_id = "DA9052_BUCK_PRO";
	}
#endif

	mxc_register_device(&mxc_rtc_device, NULL);
	mxc_register_device(&mxc_ipu_device, &mxc_ipu_data);
	mxc_register_device(&mxc_ldb_device, &ldb_data);
	mxc_register_device(&mxc_tve_device, &tve_data);
	if (!mxc_fuse_get_vpu_status())
		mxc_register_device(&mxcvpu_device, &mxc_vpu_data);
	if (!mxc_fuse_get_gpu_status())
		mxc_register_device(&gpu_device, &gpu_data);
	mxc_register_device(&mxcscc_device, NULL);
//	mxc_register_device(&pm_device, &loco_pm_data);
	mxc_register_device(&mxc_dvfs_core_device, &dvfs_core_data);
	mxc_register_device(&busfreq_device, &bus_freq_data);

	mxc_register_device(&mxc_iim_device, &iim_data);

//	mxc_register_device(&mxc_flexcan0_device, &flexcan0_data);
//	mxc_register_device(&mxc_flexcan1_device, &flexcan1_data);	

//	mxc_register_device(&mxc_pwm2_device, NULL);
//	mxc_register_device(&mxc_pwm1_backlight_device, &mxc_pwm_backlight_data);
	mxc_register_device(&mxcsdhc1_device, &mmc1_data);

	mxc_register_device(&mxc_ssi1_device, NULL);
	mxc_register_device(&mxc_ssi2_device, NULL);
	mxc_register_device(&mxc_alsa_spdif_device, &mxc_spdif_data);
	mxc_register_device(&ahci_fsl_device, &sata_data);
	mxc_register_device(&imx_ahci_device_hwmon, NULL);
	mxc_register_device(&mxc_fec_device, &fec_data);
	mxc_register_device(&mxc_ptp_device, NULL);
	/* ASRC is only available for MX53 TO2.0 */
	if (mx53_revision() >= IMX_CHIP_REVISION_2_0) {
		mxc_asrc_data.asrc_core_clk = clk_get(NULL, "asrc_clk");
		clk_put(mxc_asrc_data.asrc_core_clk);
		mxc_asrc_data.asrc_audio_clk = clk_get(NULL, "asrc_serial_clk");
		clk_put(mxc_asrc_data.asrc_audio_clk);
		mxc_register_device(&mxc_asrc_device, &mxc_asrc_data);
	}

#if defined(CONFIG_TOUCHSCREEN_ADS7846)
//	spi_register_board_info(mxc_ts_device,
//				ARRAY_SIZE(mxc_ts_device));
  //  platform_device_register(&sk_spi_gpio_device);
    spi_register_board_info(mxc_ts_device, ARRAY_SIZE(mxc_ts_device));
    platform_device_register(&sk_spi0_gpio_device);
    platform_device_register(&sk_spi1_gpio_device);
    platform_device_register(&sk_spi2_gpio_device);
#endif
	i2c_register_board_info(0, mxc_i2c0_board_info,
				ARRAY_SIZE(mxc_i2c0_board_info));
	i2c_register_board_info(1, mxc_i2c1_board_info,
				ARRAY_SIZE(mxc_i2c1_board_info));
//	i2c_register_board_info(2, mxc_i2c2_board_info,
//				ARRAY_SIZE(mxc_i2c2_board_info));

	sgtl5000_data.ext_ram_clk = clk_get(NULL, "emi_fast_clk");
	clk_put(sgtl5000_data.ext_ram_clk);
	mxc_register_device(&mxc_sgtl5000_device, &sgtl5000_data);

	spdif_audio_data.ext_ram_clk = clk_get(NULL, "emi_fast_clk");
	clk_put(spdif_audio_data.ext_ram_clk);
	mxc_register_device(&mxc_spdif_audio_device, &spdif_audio_data);

	mx5_set_otghost_vbus_func(mx53_gpio_usbotg_driver_vbus);
	mx5_usb_dr_init();
	mx5_set_host1_vbus_func(mx53_loco_usbh1_vbus);
	mx5_usbh1_init();
	mxc_register_device(&mxc_nandv2_mtd_device, &mxc_nand_data);
	mxc_register_device(&mxc_v4l2_device, NULL);
	mxc_register_device(&mxc_v4l2out_device, NULL);
//	loco_add_device_buttons();
//	pm_power_off = da9053_power_off;
//	pm_i2c_init(I2C1_BASE_ADDR - MX53_OFFSET);
}

static void __init mx53_loco_timer_init(void)
{
	struct clk *uart_clk;

	mx53_clocks_init(32768, 24000000, 0, 0);

	uart_clk = clk_get_sys("mxcintuart.0", NULL);
	early_console_setup(MX53_BASE_ADDR(UART1_BASE_ADDR), uart_clk);
}

static struct sys_timer mxc_timer = {
	.init	= mx53_loco_timer_init,
};

/*
 * The following uses standard kernel macros define in arch.h in order to
 * initialize __mach_desc_MX53_LOCO data structure.
 */
MACHINE_START(MX53_LOCO, "Freescale MX53 LOCO Board")
	/* Maintainer: Freescale Semiconductor, Inc. */
	.fixup = fixup_mxc_board,
	.map_io = mx5_map_io,
	.init_irq = mx5_init_irq,
	.init_machine = mxc_board_init,
	.timer = &mxc_timer,
MACHINE_END
